/****************************************************************************
 Module
   EventCheckers.c

 Revision
   1.0.1

 Description
   This is the sample for writing event checkers along with the event
   checkers used in the basic framework test harness.

 Notes
   Note the use of static variables in sample event checker to detect
   ONLY transitions.

 History
 When           Who     What/Why
 -------------- ---     --------
 08/06/13 13:36 jec     initial version
****************************************************************************/

// this will pull in the symbolic definitions for events, which we will want
// to post in response to detecting events
#include "ES_Configure.h"
// this will get us the structure definition for events, which we will need
// in order to post events in response to detecting events
#include "ES_Events.h"
// if you want to use distribution lists then you need those function
// definitions too.
#include "ES_PostList.h"
// This include will pull in all of the headers from the service modules
// providing the prototypes for all of the post functions
#include "ES_ServiceHeaders.h"
// this test harness for the framework references the serial routines that
// are defined in ES_Port.c
#include "ES_Port.h"
// include our own prototypes to insure consistency between header &
// actual functionsdefinition
#include "EventCheckers.h"
#include "dbprintf.h"

#include "GameControlSM.h"
#include "LeaderSM.h"

/****************************************************************************
 Function
   Check4Keystroke
 Parameters
   None
 Returns
   bool: true if a new key was detected & posted
 Description
   for testing with terminal for final project in ME218B
****************************************************************************/
#ifdef TERM_TESTING
// For testing events and HSM with keyboard inputs in place of sensors / comms
bool Check4Keystroke(void)
{
//	if (IsNewKeyReady())   // new key waiting?
//	{
//		ES_Event_t ThisEvent;
//		char ThisChar;
//
//		ThisChar = (char)GetNewKey();
//
//		if('s' == ThisChar)
//		{
//            if(Waiting == QueryLeaderSM())
//            {
//                ThisEvent.EventType = StartButtonPressed;
//                PostLeaderSM(ThisEvent);
//				PostTimerSM(ThisEvent);
//            }
//		
//		} else if('a' == ThisChar)
//		{
//			ThisEvent.EventType = TapeFound;
//			PostLeaderSM(ThisEvent);
//		} else if('r' == ThisChar)
//		{
//			ThisEvent.EventType = DoneReloading;
//			PostLeaderSM(ThisEvent);
//		} else if('l' == ThisChar)
//		{
//			ThisEvent.EventType = DoneLaunching;
//			PostLeaderSM(ThisEvent);
//		} else if('1' == ThisChar)
//		{
//			ThisEvent.EventType = LaunchPositionReached;
//			PostLeaderSM(ThisEvent);
//		} else if('2' == ThisChar)
//		{
//			ThisEvent.EventType = ReloadPositionReached;
//			PostLeaderSM(ThisEvent);
//		} else if('b' == ThisChar)
//		{
//			ThisEvent.EventType = BeaconFound;
//            ThisEvent.EventParam = 0;
//			PostLeaderSM(ThisEvent);
//		} else if('m' == ThisChar)
//		{
//			ThisEvent.EventType = MsgReceived;
//			PostLeaderSM(ThisEvent);
//		}
//	
//		return true;
//	}
//
//	else
//	{
//		return false;
//	}
    return false;
}
#endif

bool StartButtonChecker(void)
{
    static uint32_t LastStartButtonStatus = 0;
    uint32_t CurrentStartButtonStatus;
    bool ReturnVal = false;
    
    // Read status of button
    CurrentStartButtonStatus = PORTBbits.RB3;
    
    // If we are waiting for game to start
	if(Waiting == QueryLeaderSM())
	{
        //DB_printf("Start button checker\r\n");

		// If the start button value has changed and is low
		if(CurrentStartButtonStatus != LastStartButtonStatus && CurrentStartButtonStatus == 0)
		{
			// Let the leader know
            ES_Event_t ThisEvent;
			ThisEvent.EventType = StartButtonPressed;
			PostLeaderSM(ThisEvent);
			PostTimerSM(ThisEvent);
            
            ReturnVal = true;
		}
		
		LastStartButtonStatus = CurrentStartButtonStatus;
	}
    
    return ReturnVal;
}


bool ReloadButtonChecker(void)
{
    static uint32_t LastReloadButtonStatus = 0;
	static uint8_t HowManyClicks = 0;
    uint32_t CurrentReloadButtonStatus;
    bool ReturnVal = false;
    
    // Read status of button
    CurrentReloadButtonStatus = PORTBbits.RB4;

    // If we are waiting for game to start
	if(Reloading == QueryGameControlSM())
	{
        //DB_printf("Checking reload\r\n");
		// If the start button value has changed and is low
		if(CurrentReloadButtonStatus != LastReloadButtonStatus && CurrentReloadButtonStatus == 0)
		{
			if(HowManyClicks < 2)
			{
				HowManyClicks++;
			}
			
			// After two clicks, let the leader know
			if(HowManyClicks == 2)
			{
				ES_Event_t ThisEvent;
				ThisEvent.EventType = DoneReloading;
				PostLeaderSM(ThisEvent);
				
				ReturnVal = true;
			}
		}
		LastReloadButtonStatus = CurrentReloadButtonStatus;
	}
    return ReturnVal;
}