#include <xc.h>
// PIC32MX170F256B Configuration Bit Settings for ME218 operating at 40MHz from internal fast oscillator (8Mhz)

#include <stdbool.h>
#include "../ProjectHeaders/PIC32PortHAL.h"
#include "../FrameworkHeaders/bitdefs.h"

// Port A pin mask
uint32_t PORTA_PinMask = BIT0HI | BIT1HI | BIT2HI | BIT3HI | BIT4HI;

// Port B pinmask
uint32_t PORTB_PinMask =  BIT0HI | BIT1HI | BIT2HI | BIT3HI | BIT4HI | BIT5HI | BIT6HI | BIT7HI | BIT8HI | BIT9HI | BIT10HI | BIT11HI | BIT12HI | BIT13HI | BIT14HI | BIT15HI;

bool PortSetup_CheckValidity(PortSetup_Port_t WhichPort, PortSetup_Pin_t WhichPin) {

    // If port is port A
    if(WhichPort == 0) {

        // If pin is one of port A's pins
        if(WhichPin | PORTA_PinMask > PORTA_PinMask) {
            return true;
        }
    }

    // If port is port B
    if(WhichPort == 1) {

        // If pin is one of port B's pins
        if(WhichPin | PORTB_PinMask > PORTB_PinMask) {
            return true;
        }
    }

    // If port is not valid
    return false;

}

bool PortSetup_ConfigureDigitalOutputs(PortSetup_Port_t WhichPort, PortSetup_Pin_t WhichPin) {
        
    // Check if port is valid
    if(PortSetup_CheckValidity(WhichPort, WhichPin)) {

        // If port A
        if(WhichPort == 0) {

            // Set lines to digital i.e. ANSELA = 0 where WhichPin
            ANSELA = ANSELA & ~WhichPin;

            // Set lines to output i.e. TRISA = 0 where WhichPin
            TRISA = TRISA & ~WhichPin;
        }

        // If port B
        if(WhichPort == 1) {

            // Set lines to digital i.e. ANSELB = 0 where WhichPin
            ANSELB = ANSELB & ~WhichPin;

            // Set lines to output i.e. TRISB = 0 where WhichPin
            TRISB = TRISB & ~WhichPin;
        }
        return true;
    }

    // If port is invalid
    return false;
}

bool PortSetup_ConfigureDigitalInputs(PortSetup_Port_t WhichPort, PortSetup_Pin_t WhichPin) {

    // Check if port is valid
    if(PortSetup_CheckValidity(WhichPort, WhichPin)) {

        // If port A
        if(WhichPort == 0) {

            // Set lines to digital i.e. ANSELA = 0 where WhichPin
            ANSELA = ANSELA & ~WhichPin;

            // Set lines to input i.e. TRISA = 1 where WhichPin
            TRISA = TRISA | WhichPin;
        }

        // If port B
        if(WhichPort == 1) {

            // Set lines to digital i.e. ANSELB = 0 where WhichPin
            ANSELB = ANSELB & ~WhichPin;

            // Set lines to input i.e. TRISB = 1 where WhichPin
            TRISB = TRISB | WhichPin;
        }

    }
}

bool PortSetup_ConfigureAnalogInptus(PortSetup_Port_t WhichPort, PortSetup_Pin_t WhichPin) {

    // Check if port is valid
    if(PortSetup_CheckValidity(WhichPort, WhichPin)) {

        // If port A
        if(WhichPort == 0) {

            // Set lines to analog i.e. ANSELA = 1 where WhichPin
            ANSELA = ANSELA | WhichPin;

            // Set lines to input i.e. TRISA = 1 where WhichPin
            TRISA = TRISA | WhichPin;
        }

        // If port B
        if(WhichPort == 1) {

            // Set lines to analog i.e. ANSELB = 1 where WhichPin
            ANSELB = ANSELB | WhichPin;

            // Set lines to input i.e. TRISB = 1 where WhichPin
            TRISB = TRISB | WhichPin;
        }

    }
}

bool PortSetup_ConfigurePullUps(PortSetup_Port_t WhichPort, PortSetup_Pin_t WhichPin) {

    // Check if port is valid
    if(PortSetup_CheckValidity(WhichPort, WhichPin)) {

        // If port A
        if(WhichPort == 0) {

            // Set lines to digital i.e. ANSELA = 0 where WhichPin
            ANSELA = ANSELA & ~WhichPin;

            // Set lines to input i.e. TRISA = 1 where WhichPin
            TRISA = TRISA | WhichPin;

            // Set lines to pull up i.e. CNPUA = 1 where WhichPin
            CNPUA = CNPUA | WhichPin;
        }

        // If port B
        if(WhichPort == 1) {

            // Set lines to digital i.e. ANSELB = 1 where WhichPin
            ANSELB = ANSELB & ~WhichPin;

            // Set lines to input i.e. TRISB = 1 where WhichPin
            TRISB = TRISB | WhichPin;

            // Set lines to pull up i.e. CNPUB = 1 where WhichPin
            CNPUB = CNPUB | WhichPin;
        }

    }
}

bool PortSetup_ConfigurePullDowns(PortSetup_Port_t WhichPort, PortSetup_Pin_t WhichPin) {

    // Check if port is valid
    if(PortSetup_CheckValidity(WhichPort, WhichPin)) {

        // If port A
        if(WhichPort == 0) {

            // Set lines to digital i.e. ANSELA = 0 where WhichPin
            ANSELA = ANSELA & ~WhichPin;

            // Set lines to input i.e. TRISA = 1 where WhichPin
            TRISA = TRISA | WhichPin;

            // Set lines to pull down i.e. CNPDA = 1 where WhichPin
            CNPDA = CNPDA | WhichPin;
        }

        // If port B
        if(WhichPort == 1) {

            // Set lines to digital i.e. ANSELB = 1 where WhichPin
            ANSELB = ANSELB & ~WhichPin;

            // Set lines to input i.e. TRISB = 1 where WhichPin
            TRISB = TRISB | WhichPin;

            // Set lines to pull up i.e. CNPDB = 1 where WhichPin
            CNPDB = CNPDB | WhichPin;
        }

    }
}

bool PortSetup_ConfigureOpenDrain(PortSetup_Port_t WhichPort, PortSetup_Pin_t WhichPin) {

    // Check if port is valid
    if(PortSetup_CheckValidity(WhichPort, WhichPin)) {

        // If port A
        if(WhichPort == 0) {

            // Set lines to digital i.e. ANSELA = 0 where WhichPin
            ANSELA = ANSELA & ~WhichPin;

            // Set lines to output i.e. TRISA = 0 where WhichPin
            TRISA = TRISA & ~WhichPin;

            // Set lines to open drain i.e. ODCA = 1 where WhichPin
            ODCA = ODCA | WhichPin;
        }

        // If port B
        if(WhichPort == 1) {

            // Set lines to digital i.e. ANSELB = 0 where WhichPin
            ANSELB = ANSELB & ~WhichPin;

            // Set lines to output i.e. TRISB = 0 where WhichPin
            TRISB = TRISB & ~WhichPin;

            // Set lines to open drain i.e. ODCA = 1 where WhichPin
            ODCB = ODCB | WhichPin;
        }

    }
}

bool PortSetup_ConfigureChangeNotification(PortSetup_Port_t WhichPort, PortSetup_Pin_t WhichPin) {

    if(WhichPin == 0) {
        
        // Disable if port A & WhichPin is 0
        if(WhichPort == 0) {
            CNCONA = 0;
        }
        
        // Disable if port B & WhichPin is 0
        if(WhichPort == 1) {
            CNCONB = 0;
        }
    }
    
    // Check if port is valid
    if(PortSetup_CheckValidity(WhichPort, WhichPin)) {

        // If port A
        if(WhichPort == 0) {
            
            // Global change notifications enabled
            CNCONAbits.ON = 1;

            // Set lines to digital i.e. ANSELA = 0 where WhichPin
            ANSELA = ANSELA & ~WhichPin;

            // Set lines to input i.e. TRISA = 1 where WhichPin
            TRISA = TRISA | WhichPin;

            // Set lines to enable change notification i.e. CNENA = 1 where WhichPin
            CNENA = CNENA | WhichPin;
        }

        // If port B
        if(WhichPort == 1) {

            // Global change notifications enabled
            CNCONBbits.ON = 1;
            
            // Set lines to digital i.e. ANSELB = 1 where WhichPin
            ANSELB = ANSELB & ~WhichPin;

            // Set lines to input i.e. TRISB = 1 where WhichPin
            TRISB = TRISB | WhichPin;

            // Set lines to enable change notification i.e. CNENB = 1 where WhichPin
            CNENB = CNENB | WhichPin;
            
            CNPDB = CNPDB | WhichPin;
        }

    }
}
